! ---------------------------------------------------------------------------- !
!   ScenicI.h, traduzione di Scenic.h
!       corrected 3.1 Jul00 by Roger Firth and Stefano Gaburri
!       extended  3.0 Jul00 by Roger Firth and Stefano Gaburri (EXAMINE room)
!       extended  2.5 May00 by Roger Firth and Stefano Gaburri
!       extended  2.4 May00 by Roger Firth and Stefano Gaburri
!       extended  2.3 May00 by Roger Firth (compatibility with glulx)
!       extended  2.2 Mar00 by Roger Firth
!       extended  2.1 Mar00 by Roger Firth with input from Paul E Bell
!       reworked  2.0 Mar00 by Roger Firth for Inform 6
!       extended  1.1 Sep95 by Joe Mason
!       original  1.0 Jan95 by Richard Barnett for Inform 5
!
!       Adattato alla lingua italiana da Paolo Maroncelli 030104
!       traduzione commenti e istruzioni di Paolo Lucchesi 021010
!
!
! ---------------------------------------------------------------------------- !
!   Installatione: aggiungere la linea
!
!       Include "ScenicI";
!
!   verso la fine del gioco, DOPO aver usato la proprieta' Scenic, ma PRIMA
!   dell'istruzione Include "ItalianG". Opzionalmente si puo' premettere la riga
!
!       Global ScenicFlag 0 [+ 1] [+ 2] [+ 4];
!
!   per controllare l'elaborazione alla fine del turno e la possibilita'
!   di 'esaminare' le stanze:
!       0 : sia "ESAMINA swd" che "ALTROVERBO swd" contano come turno
!           (dove 'swd' e' una parola nel dizionario definita nella proprieta'
!               Scenic di una stanza),
!           "EXAMINE room" e' gestito tradizionalmente.
!        
!      +1 = "ALTROVERBO swd" non conta come turno.
!      +2 = "ESAMINA swd" non conta come turno.
!      +4 = "ESAMINA stanza" produce la descrizione della stanza.
!
!   E' opzionale anche il controllo sul messaggio "E' solo scenografia.", che
!   appare quando viene compiuta un'azione diversa da ESAMINA|CONSULTA|GUARDA
!   a un elemento di scenografia. Potete premettere alla riga di inclusione
!   qualcosa come
!
!       Constant ScenicError "Non danneggiare la scenografia!";
!
!   (per usare un messaggio personalizzato) o
!
!       [ ErroreScenico; switch(random(3)) {
!           1: "E' solo scenografia.";
!           2: "Non danneggiare la scenografia!";
!           default: "Niente da fare."; } ];
!       Constant ScenicError ErroreScenico;
!
!   (per aggiungere flessibilita' con una routine) o semplicemente
!
!       Constant ScenicError NULL;
!
!   (per avere il tradizionale "Non e' importante ai fini del gioco".)
!
!   ATTENZIONE: La libreria implementa una versione di ParseError() (una Entry
!   Point Routine). Se questa e' gia' stata definita, magari in un'altra
!   libreria, l'errore (ingannevole) di compilazione e':  Expected routine 
!   name but found ParserError
!
! ---------------------------------------------------------------------------- !
!   Questa libreria permette di ESAMINARE stanze e non-oggetti scenici.
!   Queste due funzionalita' sono indipendenti (anche se collegate).
!
!   1. descrizione delle STANZE
!
!   Un modo standard di evitare risposte insensate come "Non vedi nulla del
!   genere" e' quello di aggiungere una proprieta' name agli oggetti stanza.
!   Per esempio si puo' codificare una foresta usando questa tecnica:
!
!   Object  foresta "Foresta"
!     with  description "La foresta...",
!           name 'foresta' 'bosco',
!           etc... ;
!
!   Ora, ESAMINA LA FORESTA mostrera' la piu' amichevole risposta "Non e'
!   importante ai fini del gioco". Questa libreria fa' un ulteriore passo
!   avanti: una volta che la funzionalita' viene attivata, ESAMINA LA FORESTA
!   stampera' la descrizione della foresta (come per GUARDA, ma senza far
!   menzione degli oggetti presenti nella stanza).
!
!   2. Descrizioni SCENICHE
!
!   E' probabile che la descrizione della foresta menzioni alberi, cespugli
!   a altro, ed e' altrettanto probabile che il giocatore vorra' ESAMINARLI
!   tutti. I non-oggetti scenici sono un qualcosa che sta a meta' strada fra
!   la mera esistenza di una parola che puo' essere scritta per sentirsi
!   rispondere con un messaggio "Non e' importante...", e la fatica di creare
!   un distinto oggetto di scenografia la cui sola caratteristica realmente
!   interessante e' la descrizione.
!
!   Ovvero, piuttosto che la minimalistica soluzione:
!
!       Object  foresta "Foresta"
!         with  description "La foresta...",
!               name 'foresta' 'bosco' 'albero' 'alberi' 'cespuglio' 
!                     'cespugli' 'sottobosco'
!               etc... ;
!
!   o la pesante e complessa soluzione:
!
!       Object  foresta "Foresta"
!         with  description "La foresta...",
!               name 'foresta' 'bosco',
!               etc... ;
!       Object  -> "Alberi"
!         with  description "Gli alberi troneggiano sopra di te.",
!               name 'albero' 'alberi',
!         has   scenery;
!       Object  -> "Cespugli"
!         with  description "I cespugli sono folti.",
!               name 'cespuglio' 'cespugli',
!         has   scenery;
!       Object  -> "Sottobosco"
!         with  description "Il verde sottobosco.",
!               name 'sottobosco',
!         has   scenery;
!
!   ora e' possibile riunire tutto con:
!
!       Object  foresta "Foresta"
!         with  description "La foresta...",
!               name 'foresta' 'bosco',
!               scenic  'albero' 'trees' 0 "Gli alberi troneggiano sopra di te."
!                       'cespuglio' 'cespugli' 0 "I cespugli sono folti."
!                       'sottobosco' 0 "Il verde sottobosco.",
!               etc... ;
!
!   o, per variare casualmente le descrizioni sceniche:
!
!       [ DescriviAlberi;   ... codice per stampare una descrizione ...; ];
!       [ DescriviCespugli; ... codice per stampare una descrizione ...; ];
!       [ DescriviSottob;   ... codice per stampare una descrizione ...; ];
!       Object  foresta "Foresta"
!         with  description "La foresta...",
!               name 'foresta' 'bosco',
!               scenic  'albero' 'alberi' 0 DescriviAlberi
!                       'cespuglio' 'cespugli' 0 DescriviCespugli
!                       'sottobosco' 0 DescriviSottob,
!               etc... ;
!
!   o, finalmente, per avere il tradizionale "Non e' importante..."
!
!       Object  foresta "Foresta"
!         with  description "La foresta...",
!               name 'foresta' 'bosco',
!               scenic  'albero' 'alberi' 'cespuglio' 'cespugli'
!                       'sottobosco' 0 NULL,
!               etc... ;
!
!   3. SOMMARIO
!
!   Fidatevi: tutto cio' e' piu' semplice di quanto sembri. Seguite questi
!   suggerimenti:
!   
!   - settate la costante ScenicFlag a 4, per attivare la possibilita' di
!     ESAMINARE le stanze.
!   - definite per ogni stanza la proprieta' NAME assegnandoli solo il nome
!     della stanza (e eventuali sinonimi)
!   - usate la proprieta' SCENIC con stringhe per oggetti secondari che
!     vengono menzionati nella descrizione della stanza
!   - usate la proprieta' SCENIC con NULL per oggetti terziari che vengono
!     menzionati nelle descrizioni degli oggetti secondati.
!   Per esempio:
!
!       Object  foresta "Foresta"
!         with  description
!                   "La foresta sembra stendersi in ogni direzione. Attorno a 
!                   te i cespugli, intricati in modo quasi impenetrabile, 
!                   silenziosamente combattono per il possesso del sottobosco,
!                   mentre sopra di te gli alberi si stagliano alti e spettrali
!                   attraverso l'aria umida e soffocante.",
!               name 'foresta' 'bosco',
!               scenic
!                   'albero' 'alberi' 'alti' 'spettrali' 0 "Ricoperti di
!                     licheni, gli alberi non offrono nessun appiglio oltre
!                     ai rami che si trovano molti metri sopra di te."
!                   'cespugli' 'cespuglio' 'intricati' 0 "Rovi e spine spuntano
!                     dai cespugli avviluppandosi in pericolose spire."
!                   'sottobosco' 'terreno' 0 "L'unico colore vivo in questo
!                     scenario desolato. Il sottobosco verde smeraldo e' soffice
!                     sotto i tuoi piedi."
!                   'aria' 'umida' 'soffocante' 0 "Densa, quasi soffocante,
!                     l'atmosfera pesante ti opprime."
!                   'cielo' 0 "Il cielo non e' visibile attraverso le foglie."
!                   'lichene' 'licheni' 'ramo' 'rami' 'spine' 'spina' 'rovo'
!                   'rovi' 'spire' 'spira' 'colore' 'verde' 'smeraldo' 'foglia'
!                   'foglie' 'atmosfera' 0 NULL,
!               etc... ;
!
!
!
! ---------------------------------------------------------------------------- !
#ifdef SCENIC;
#ifdef DEBUG; message "Compiling ScenicI.h"; #endif;

! Compatibilta' con glulx (che usa parole di quattro byte)
#ifndef WORDSIZE;
    Constant TARGET_ZCODE 0;
    Constant WORDSIZE 2+TARGET_ZCODE;   ! evita il warning in compilazione
#endif;

! Flag che controlla le opzioni
#ifndef ScenicFlag;
Global  ScenicFlag = 0;
#endif;

! Il messaggio per "ALTROVERBO swd".
Default ScenicError "E' solo scenografia.";

[ GetScenicDesc obj swd x y z;
        ! Se obj ha la proprieta' 'scenic', controlla se questa include swd.
        ! Se cosi' e', ritorna l'argomento associato, altrimenti rfalse.
        if (~~(obj provides scenic && swd)) rfalse;
        x = obj.&scenic; y = x + obj.#scenic; z = 0;
        for ( : x<y : x=x+WORDSIZE) switch (z) {
            0:  if (0 == x-->0) z = z + 2; if (swd == x-->0) z++;
            1:  if (0 == x-->0) z = z + 2;
            2:  z = 0;
            3:  return x-->0;
            }
        rfalse; ];

Global  ScenicWord;
        ! Una parola di dizionario che potrebbe essere menzionata in una
        ! proprieta' 'scenic'. Una volta trovata, viene messa a false per
        ! arrestare la ricerca.

[ ScenicPrintOrRun x y;
        ! Stampa o esegue (a seconda che sia stringa o routine) il messaggio
        ! di errore di sceneria, oppure (se e' NULL) stampa il messaggio
        ! standard "Non e' importante...". Se necessarrio esegue le normali
        ! operazioni di fine turno di Inform.
        ScenicWord = false;
        !SetPronoun('it', NULL); SetPronoun('them', NULL); 
        SetPronoun('esso', NULL); SetPronoun('essa', NULL); 
        SetPronoun('essi', NULL); SetPronoun('esse', NULL);
        ! (Modificato nell'adattamento)
        if (x == NULL) L__M(##Miscellany, 39);
        else { if (metaclass(x) == Routine) x(); else print (string) x, "^"; }
        if (~~(ScenicFlag & y)) InformLibrary.end_turn_sequence();
        ];

[ CheckScenicExamine obj x;
        ! Se obj ha una proprieta' 'scenic', controlla se include ScenicWord.
        ! Se e' cosi', stampa/esegue la descrizione.
        x = GetScenicDesc(obj, ScenicWord); if (x) {
            ScenicPrintOrRun(x, $0002);
            }
        ];

[ CheckScenicOther obj x;
        ! Se obj ha una proprieta' 'scenic', controlla se include ScenicWord.
        ! Se e' cosi', stampa/esegue la risposta appropriata.
        x = GetScenicDesc(obj, ScenicWord); if (x) {
            if (x ~= NULL) x = ScenicError;
            ScenicPrintOrRun(x, $0001);
            }
        ];

[ Handle_CANTSEE_PE;
        ! Per un errore CANTSEE, setta ScenicWord all'ultima parola della
        ! linea, poi cerca quella parola in ogni propriet 'scenic' della
        ! stanza, e poi per ogni oggetto in scope. Per ogni operazione di
        ! tipo ESAMINA, mostra la descrizione scenica.
        do ScenicWord = NextWordStopped(); until (ScenicWord == -1);
        wn = wn - 2; ScenicWord = NextWord();
        if (ScenicWord) {
            if (action_to_be == ##Examine or ##Search or ##Consult) {
                CheckScenicExamine(location);
                if (ScenicWord) LoopOverScope(CheckScenicExamine);
                }
            else {
                CheckScenicOther(location);
                if (ScenicWord) LoopOverScope(CheckScenicOther);
                }
            if (~~ScenicWord) rtrue;
            }
        rfalse;                     ! Gli errori non sono gestiti.
        ];

[ Handle_SCENERY_PE;
        ! Per un errore di SCENERY (usare una parola della proprieta' NAME
        ! della stanza) dovuto ad un'operazione di tipo EXAMINE, mostra la
        ! descrizione della stanza.
        if (ScenicFlag & $0004) {
            if (action_to_be == ##Examine or ##Search or ##Consult) {
                if (location.describe ~= NULL) RunRoutines(location, describe);
                else {
                    if (location.description == 0) RunTimeError(11, location);
                    else PrintOrRun(location, description);
                    }
                InformLibrary.end_turn_sequence();
                rtrue;
                }
            }
        rfalse;                     ! Gli errori non sono gestiti.
        ];

[ ParserError eType;
        ! Entry point standard, chiamato dalle routine di libreria dopo
        ! un errore di parsing.
        switch (eType) {
            CANTSEE_PE: return Handle_CANTSEE_PE();
            SCENERY_PE: return Handle_SCENERY_PE();
            default:    rfalse;     ! Error was NOT handled by this routine.
            }
        ];

! ---------------------------------------------------------------------------- !

#ifdef DEBUG;
[ RoomExOnSub;  ScenicFlag = ScenicFlag | $0004; "EXAMINE per le stanze abilitato."; ];
[ RoomExOffSub; ScenicFlag = ScenicFlag & $FFFB; "EXAMINE per le stanze disabilitato."; ];
[ RoomExSub;    if (ScenicFlag & $0004) RoomExOffSub (); else RoomExOnSub (); ];

Verb meta 'roomex' *           -> RoomEx
                   * 'on'      -> RoomExOn
                   * 'off'     -> RoomExOff;
#endif;

#endif;
! ---------------------------------------------------------------------------- !
