!-----------------------------------------------------------------------------
! Magic and Fight System --- Magical objects
! For INFORM compiler and Z-Machine text based Adventure games
!-----------------------------------------------------------------------------
!  Author: Giancarlo Niccolai
!  Version: 1.3
!  Last Revised: 2002-11-24
!  Contact author at: giancarlo@niccolai.ws
!  Feel free to use and distribute this file, but please report me if
!  you intend to change it (in better), so we can upgrade together
!
!  Released under GPL 2.0, or any later version at your will
!
!-----------------------------------------------------------------------------
! Some objects are reqired for the standard spells. You will have to include
! this library before the standard spells and attack spells lib (or use the
! MFS.h file that includes correctly all libs) to have the spells using the
! objects ready and available.
!------------------------------------------------------------------------------

!----------------------------------------------------------------
!Writable Scroll, a standard writable.
!----------------------------------------------------------------

class Writable_Scroll(20) class Scroll
   with
   name "pergamena" "vuota", article "una",
   description "Questa pergamena vuota pu@`o essre scritta con vari incantesimi.",
   content filfre_spell 0 0 0 0 0 0,

   after [;
      Scribe:
            self.description = "La percamena contiene ora alcuni incantesimi.";
            rfalse;
   ];

!----------------------------------------------------------------
!An object that can be weared and gives a bouns
!----------------------------------------------------------------

Class ClothingBonus
with
   ! bonuses for mages
   time_bonus,
   power_bonus,
   cost_bonus,

   ! bonuses for warriors
   attack,
   defense,
   armor,
   base_damage,
   min_damage,
   max_damage,
   initiative,

   ! Messages
   wear_message "Ti senti in qualche modo pi@`u forte.",
   disrobe_message "Ti senti in qualche modo pi@`u debole.",

   giving_bonus [;
      if ( self has worn ) rtrue;
      rfalse;
   ],

   before [;
      Wear:
         if ( self hasnt worn ) {
            give self worn;
            print (string) self.wear_message,"^";
            rtrue;
         }
      Disrobe:
         if ( self has worn ) {
            give self ~worn;
            print (string) self.wear_message,"^";
         }
         rtrue;
   ],

   after [;
      Examine:
         DescBonusGiver(self);
   ],

   has clothing;

!-----------------------------------------------------------------------
! Good taste food
!-----------------------------------------------------------------------

class GoodFood
with
   health_effect 1,

   before [;
      Smell:
         "Si direbbe buono. Anzi, molto buono.^";
   ],

   after [;
      Eat:
         print "MMMM.... Era davvero buono!^";
         if (player provides heal) player.heal(self.health_effect);
         rtrue;
   ],
   has edible;

class HealingPotion
with
   name "fiala" "pozione" "liquido",
   health_effect 6,
   doses 1,
   liquid_desc "verde",
   smell_desc "Ha un forte odore di menta",

   description [;
      print "@`E una fiala decorata riccamente, ";
      if (self.doses == 0)
         "attualmente vuota.";
      print "che contiene un liquido ",(string) self.liquid_desc;
      print ". Dovrebbe";
      if (self.doses ~= 1) print "ro";
      print " essere rimast";
      if (self.doses ~= 1) "e circa " , (number) self.doses, " dosi.";
      else "a una dose.";
   ],

   before [;
      Drink:
         if (self.doses == 0 )
            "La fiala @`e vuota! Non  rimasto nulla della pozione.";
         print "MMMM.... Aveva un buon sapore!";
         self.doses = self.doses-1;
         if (self.doses == 0) print " @`E un peccato che sia finita.^";
         else print "^";

         if (player provides heal) player.heal(self.health_effect);
         rtrue;
      Smell:
         print_ret (string) self.smell_desc, ".";
      Open:
         "Perch@'e non dici solo che vuoi berla?";
   ]
   has female;


!-------------------------------------------------------
! A golem being able to fight!
!-------------------------------------------------------
ifdef Follower;
ifdef NPC_Warrior;

Object golem "golem" class NPC_Warrior Follower
with
   name 'golem' 'statua' 'di' 'pietra',
   hands_name "con il suo pugno possente",
   min_damage 5,
   max_damage 20,
   base_damage 5,
   armor 6,
   max_health 65,
   health 65,
   initiative 0,
   attack 6,
   defense 2,

   will_follow 1,
   follow_ok_msg[;
      "Odi una profonda voce provenire dall'interno della pietra.^~Ti obbedir@`o, mio Signore!~";
   ],

   stop_ok_message [;
      "La bassa, bursca voce del golem risponde:
         ~I tuoi comandi saranno ubbiditi, Signore.~";
   ],
   initial "Una possente statua di pietra @`e qui.",
   description "@`E una grande statua di pietra, che rappresenta un uomo impressionante;
      parrebbe immobile, in questo momento. E' alta circa due metri e mezzo, e le sue
      spalle misurano pi@`u di un metro. Immagini che sia una creatura davvero forte.",

   orders [;
      if ( ~~ self.NPC_Warrior::orders() )
         if ( ~~ self.Follower::orders() )
            self.subOrders();
      rtrue;
   ],

   dead_proc [;
      self.stop_follow();
      remove self;
      "L'ultimo colpo @`e fatale perl il golem; il suo immenso colpo di
      pietra si sbriciola sotto il suo stesso peso, diventando polvere
      subito spazzata via da un vento che non sposta null'altro.";
   ],

   before [;
      Take: "@`E troppo grosso.";
   ],
   
   subOrders [ oldloc;
      Enter:
         actor = self;
         oldloc = parent( self );
         <Enter noun>;
         if ( oldloc ~= parent( self ) )
            self.who_follow = 0;
         rtrue;

      NotUnderstood: "Il golem resta silenziosamente immobile; probabilmente,
         non @`e in grado di compredere qualcosa di cos@`i complesso.";
      default:
         "Nonostante il golem voglia seguire i tuoi ordini, non ne @`e capace.";
   ];

endif;
endif;
!--------------------------------------------------------------------
! A portal leading somwhere else
!--------------------------------------------------------------------

Class MagicalPortal(3)
with
   other_place,

   short_name "portale magico",
   name 'portale' 'cancello' 'magico',
   initial "In mezzo a questa zona si trova un luminoso portale ad arco. In esso, puoi vedere
      immagini provenienti da altri luoghi.",

   description
      "Il portale ad arco @`e costituito da una strana pietra fluorescente; @`e alto circa tre metri,
      e largo due. La sua chiave di volta riporta scolpita l'immagine di una clessidra... e,
      sorprendentemente, puoi vedere il rilievo cambiare, come se ci fosse della sabbia che cade.^
      All'interno, vedi scene provenienti da un'altro luogo.",

   before 	[;
      Search:
         print "^Il portale mostra una scena distante...^";
         print "Vedi ";
         style bold;
         print (the) self.other_place;
         style roman;
         print ".^^";

         if (metaclass(self.other_place.description) == String)
            print (string) self.other_place.description;
         else
            self.other_place.description();
         print "^";
         rtrue;

      Enter:
         if ( actor == player ) {
            print "^Come entri nel portale, ti soggiunge una strana sensazione. @`E come se
               il tuo corpo fosse sbriciolato in piccoli pezzi, e riassemblato poco dopo.
               Guardandoti intorno, vedi che non sei pi@`u ", (artin) location, ".^";
            PlayerTo(self.other_place);
         }
         else {
            print "Come ", (the) actor, " entra nel portale, un lampo di luce squarcia l'aria.
               Un istante dopo di accorgi che ", (the) actor, " non @`e pi@`u qui.^";
            move actor to self.other_place;
         }
         rtrue;

      ThrownAt: Receive:
         move noun to self.other_place;
         print (The) noun, " sparisce nel portale.^";
         rtrue;
   ],
   has enterable light container open static;

!------------------------------------------------------------
! A magical pen capable to write onto a spell device
!
ifdef MagicalWritingDevice;
MagicalWritingDevice magic_pen "penna magica"
   with
   name "penna" "magica" "bianca",
   initial "Qui vedi una grande penna bianca.",
   description
      "La lunga Penna Magica ha un piumaggio bianco con rifrazioni multicolori. Muovendola
      leggermente, essa produce un suono appena percettibile, come mille piccole campane
      che suonino all'unisono. La magia che si irradia dalla penna @`e cos@`i chiara che
      @`e visibile anche alla luce del giorno.^@`E certo un oggetto magico di grande potere.",
   has female;
Endif;

