!-----------------------------------------------------------------------------
! Magic And Fight System --- TopicParse, a general topic parsing routine
! For INFORM compiler and Z-Machine text based Adventure games
!-----------------------------------------------------------------------------
!  Author: Giancarlo Niccolai
!  Version: 1.3
!  Last Revised: 2002-11-24
!  Contact author at: giancarlo@niccolai.ws
!  Feel free to use and distribute this file, but please report me if
!  you intend to change it (in better), so we can upgrade together
!
!  Released under GPL 2.0, or any later version at your will
!
!-----------------------------------------------------------------------------
! Wandering around
!-----------------------------------------------------------------------------

Ifndef MFS_Wanderer;

Constant MFS_WanderRandom = 0;
Constant MFS_WanderJump = 2;
Constant MFS_WanderAlong = 1;
Constant MFS_WanderLimit = -1;

Class MFS_Wanderer
private
   direction 0,
   pos 0,
   resting 0,
   decision 0,
   getDirAt[ rp room;
      room = parent( self );
      rp++;
      if ( self.canGo( room.n_to ) ) rp --;
      if ( rp == 0 ) return room.n_to;
      if ( self.canGo( room.ne_to ) ) rp --;
      if ( rp == 0 ) return room.ne_to;
      if ( self.canGo( room.e_to ) ) rp --;
      if ( rp == 0 ) return room.e_to;
      if ( self.canGo( room.se_to ) ) rp --;
      if ( rp == 0 ) return room.se_to;
      if ( self.canGo( room.s_to ) ) rp --;
      if ( rp == 0 ) return room.s_to;
      if ( self.canGo( room.sw_to ) ) rp --;
      if ( rp == 0 ) return room.sw_to;
      if ( self.canGo( room.w_to ) ) rp --;
      if ( rp == 0 ) return room.w_to;
      if ( self.canGo( room.nw_to ) ) rp --;
      if ( rp == 0 ) return room.nw_to;
      if ( self.canGo( room.u_to ) ) rp --;
      if ( rp == 0 ) return room.u_to;
      if ( self.canGo( room.d_to ) ) rp --;
      if ( rp == 0 ) return room.d_to;
      return nothing;
   ],

   countDirs[ rp room;
      room = parent( self );
      rp = 0;
      if ( self.canGo( room.n_to ) ) rp ++;
      if ( self.canGo( room.ne_to ) ) rp ++;
      if ( self.canGo( room.e_to ) ) rp ++;
      if ( self.canGo( room.se_to ) ) rp ++;
      if ( self.canGo( room.s_to ) ) rp ++;
      if ( self.canGo( room.sw_to ) ) rp ++;
      if ( self.canGo( room.w_to ) ) rp ++;
      if ( self.canGo( room.nw_to ) ) rp ++;
      if ( self.canGo( room.u_to ) ) rp ++;
      if ( self.canGo( room.d_to ) ) rp ++;
      return rp;
   ],

   !Find next room along a path
   getNextRoom [ room;
      ! change path orientation
      if ( self.pos == 0 )
         self.direction = 1;
      else if ( self.pos >= self.#path/2 -1 )
         self.direction = -1;

      !get next room in path
      room = self.&path-->self.pos;
      self.pos = self.pos + self.direction;
      return room;
   ],

   isNearby[ obj room;
      if ( parent( obj ) ~= nothing )
         obj = parent( obj );
      room = parent( self );
      if ( obj == room.n_to ) return s_to;
      if ( obj == room.ne_to ) return sw_to;
      if ( obj == room.e_to ) return w_to;
      if ( obj == room.se_to ) return nw_to;
      if ( obj == room.s_to ) return n_to;
      if ( obj == room.sw_to ) return ne_to;
      if ( obj == room.w_to ) return e_to;
      if ( obj == room.nw_to ) return se_to;
      if ( obj == room.u_to ) return d_to;
      if ( obj == room.d_to ) return u_to;
      return nothing;
   ],

with
   path,
   move_at_random MFS_WanderAlong,
   rest_turns 0,

   arrive [; print "^", (The) self, " arriva...^";],
   leave [; print "^",(The) self, " va via...^"; ],
   stay_here [; print "^",(The) self, " sta qui...^"; ],
   nearby [ dir; print "^",(The) self, " lurka a ", LanguageDirection( dir ),".^";],
   coming [ dir;
      print "^",(The) self, " sta arrivando da ";
      LanguageDirection(dir);
      print ".^";
   ],

   canGo [ room i;
      for ( i = 0: i < self.#path/2: i++ ) {
         if ( self.&path-->i == room ) {
            if ( self.move_at_random == MFS_WanderLimit )
               rfalse;
            rtrue;
         }
      }
      if ( self.move_at_random == MFS_WanderLimit )
         rtrue;
      rfalse;
   ],

   wander [ dest rp old;
      dest = nothing;
      ! decision must be made with respect to the previous decision
      if ( self.decision ) {
         old = parent( self );
         move self to self.decision;
      }

      switch( self.move_at_random ) {
         MFS_WanderLimit: MFS_WanderRandom:
            rp = self.countDirs();
            if ( rp ) {
               rp = random( rp )-1;
               dest = self.getDirAt( rp );
            }

         MFS_WanderAlong:
            dest = self.getNextRoom();
         MFS_WanderJump:
            rp = random( self.#path/2 );
            dest = self.&path-->rp;
      }
      ! decision must be made with respect to the previous decision
      if ( self.decision ) {
         move self to old;
      }
      if ( dest == nothing ) rfalse;

      if ( self.decision ) {
         rp = parent( self );
         if ( rp ~= self.decision ) {
            if ( player in rp )
               self.leave();
            move self to self.decision;
            if ( player in self.decision )
               self.arrive();
            self.decision = dest;
            rtrue;
         }
      }
      self.decision = dest;
      rfalse;
   ],

   start [;
      self.resting = 0;
      self.pos = 0;
      self.wander(); ! creates the first decision
      StartDaemon( self );
   ],

   stop [; StopDaemon( self ); ],

   daemon [ dir begin_in;
      if (metaclass( self.rest_turns ) ~= Routine )
         self.resting++;
      begin_in = parent( self );
      if ( (metaclass( self.rest_turns ) ~= Routine && self.resting > self.rest_turns) ||
         (metaclass( self.rest_turns ) == Routine && self.resting ) ) {
         self.wander();
         self.resting = 0;
         ! at this moment, I could have been removed
         if ( parent( self ) == nothing) return;
      }
      else {
         if ( parent( self ) == parent( player ) )
            self.stay_here();
      }
      if (metaclass( self.rest_turns ) == Routine )
         self.resting = ~~self.rest_turns();

      if ( begin_in ~= parent( player ) ) {
         dir = self.isNearby( player );
         if ( dir ) {
            ! decided to move toward player && moving next turn?
            if ( self.decision == parent( player ) &&
               ( metaclass( self.rest_turns ) ~= Routine && self.resting == self.rest_turns ||
                  metaclass( self.rest_turns ) == Routine && self.resting )
               )
                  self.coming( dir );
            else
               self.nearby( dir );
         }
      }
   ];


Endif;

! End of MFS_Wander.h
